@@ -48,6 +48,7 @@ enum {
   PROP_0,
   PROP_MODULE_NAME,
   PROP_PATH,
+  PROP_SYMBOL,
   PROP_RESIDENT,
   PROP_LOCAL_LINKAGE,
   N_PROPERTIES
@@ -70,6 +71,7 @@ struct _PeasObjectModulePrivate {
 
   gchar *path;
   gchar *module_name;
+  gchar *symbol;
 
   guint resident : 1;
   guint local_linkage : 1;
@@ -82,32 +84,48 @@ G_DEFINE_TYPE_WITH_PRIVATE (PeasObjectModule,
 #define GET_PRIV(o) \
   (peas_object_module_get_instance_private (o))
 
+#define TYPE_MISSING_PLUGIN_INFO_PROPERTY (G_TYPE_FLAG_RESERVED_ID_BIT)
+
+static const gchar *intern_plugin_info = NULL;
+
 static gboolean
 peas_object_module_load (GTypeModule *gmodule)
 {
   PeasObjectModule *module = PEAS_OBJECT_MODULE (gmodule);
   PeasObjectModulePrivate *priv = GET_PRIV (module);
-  GModuleFlags flags = 0;
-  gchar *path;
 
-  if (priv->local_linkage)
-    flags = G_MODULE_BIND_LOCAL;
+  g_return_val_if_fail (priv->module_name != NULL, FALSE);
 
-  path = g_module_build_path (priv->path, priv->module_name);
-  g_return_val_if_fail (path != NULL, FALSE);
+  if (priv->path == NULL)
+    {
+      g_return_val_if_fail (priv->resident, FALSE);
+      g_return_val_if_fail (!priv->local_linkage, FALSE);
 
-  /* g_module_build_path() will add G_MODULE_SUFFIX to the path,
-   * however g_module_open() will only try to load the libtool archive
-   * if there is no suffix specified. So we remove G_MODULE_SUFFIX here
-   * which allows uninstalled builds to load plugins as well, as there
-   * is only the .la file in the build directory.
-   */
-  if (G_MODULE_SUFFIX[0] != '\0' && g_str_has_suffix (path, "." G_MODULE_SUFFIX))
-    path[strlen (path) - strlen (G_MODULE_SUFFIX) - 1] = '\0';
+      priv->library = g_module_open (NULL, 0);
+    }
+  else
+    {
+      gchar *path;
+      GModuleFlags flags = 0;
 
-  /* Bind symbols immediately to avoid errors long after loading */
-  priv->library = g_module_open (path, flags);
-  g_free (path);
+      path = g_module_build_path (priv->path, priv->module_name);
+
+      /* g_module_build_path() will add G_MODULE_SUFFIX to the path,
+       * however g_module_open() will only try to load the libtool archive
+       * if there is no suffix specified. So we remove G_MODULE_SUFFIX here
+       * which allows uninstalled builds to load plugins as well, as there
+       * is only the .la file in the build directory.
+       */
+      if (G_MODULE_SUFFIX[0] != '\0' && g_str_has_suffix (path, "." G_MODULE_SUFFIX))
+        path[strlen (path) - strlen (G_MODULE_SUFFIX) - 1] = '\0';
+
+      if (priv->local_linkage)
+        flags = G_MODULE_BIND_LOCAL;
+
+      /* Bind symbols immediately to avoid errors long after loading */
+      priv->library = g_module_open (path, flags);
+      g_free (path);
+    }
 
   if (priv->library == NULL)
     {
@@ -118,12 +136,11 @@ peas_object_module_load (GTypeModule *gmodule)
     }
 
   /* Extract the required symbol from the library */
-  if (!g_module_symbol (priv->library,
-                        "peas_register_types",
+  if (!g_module_symbol (priv->library, priv->symbol,
                         (gpointer) &priv->register_func))
     {
-      g_warning ("Failed to get 'peas_register_types' for module '%s': %s",
-                 priv->module_name, g_module_error ());
+      g_warning ("Failed to get '%s' for module '%s': %s",
+                 priv->symbol, priv->module_name, g_module_error ());
       g_module_close (priv->library);
 
       return FALSE;
@@ -134,8 +151,8 @@ peas_object_module_load (GTypeModule *gmodule)
    */
   if (priv->register_func == NULL)
     {
-      g_warning ("Invalid 'peas_register_types' in module '%s'",
-                 priv->module_name);
+      g_warning ("Invalid '%s' in module '%s'",
+                 priv->symbol, priv->module_name);
       g_module_close (priv->library);
 
       return FALSE;
@@ -190,6 +207,7 @@ peas_object_module_finalize (GObject *object)
 
   g_free (priv->path);
   g_free (priv->module_name);
+  g_free (priv->symbol);
   g_array_unref (priv->implementations);
 
   G_OBJECT_CLASS (peas_object_module_parent_class)->finalize (object);
@@ -212,6 +230,9 @@ peas_object_module_get_property (GObject    *object,
     case PROP_PATH:
       g_value_set_string (value, priv->path);
       break;
+    case PROP_SYMBOL:
+      g_value_set_string (value, priv->symbol);
+      break;
     case PROP_RESIDENT:
       g_value_set_boolean (value, priv->resident);
       break;
@@ -243,6 +264,9 @@ peas_object_module_set_property (GObject      *object,
     case PROP_PATH:
       priv->path = g_value_dup_string (value);
       break;
+    case PROP_SYMBOL:
+      priv->symbol = g_value_dup_string (value);
+      break;
     case PROP_RESIDENT:
       priv->resident = g_value_get_boolean (value);
       break;
@@ -261,6 +285,8 @@ peas_object_module_class_init (PeasObjectModuleClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GTypeModuleClass *module_class = G_TYPE_MODULE_CLASS (klass);
 
+  intern_plugin_info = g_intern_static_string ("plugin-info");
+
   object_class->set_property = peas_object_module_set_property;
   object_class->get_property = peas_object_module_get_property;
   object_class->finalize = peas_object_module_finalize;
@@ -286,6 +312,15 @@ peas_object_module_class_init (PeasObjectModuleClass *klass)
                          G_PARAM_CONSTRUCT_ONLY |
                          G_PARAM_STATIC_STRINGS);
 
+  properties[PROP_SYMBOL] =
+    g_param_spec_string ("symbol",
+                         "Symbol",
+                         "The registration symbol to use for this module",
+                         "peas_register_types",
+                         G_PARAM_READWRITE |
+                         G_PARAM_CONSTRUCT_ONLY |
+                         G_PARAM_STATIC_STRINGS);
+
   properties[PROP_RESIDENT] =
     g_param_spec_boolean ("resident",
                           "Resident",
@@ -330,6 +365,9 @@ peas_object_module_new (const gchar *module_name,
                         const gchar *path,
                         gboolean     resident)
 {
+  g_return_val_if_fail (module_name != NULL && *module_name != '\0', NULL);
+  g_return_val_if_fail (path != NULL && *path != '\0', NULL);
+
   return PEAS_OBJECT_MODULE (g_object_new (PEAS_TYPE_OBJECT_MODULE,
                                            "module-name", module_name,
                                            "path", path,
@@ -344,7 +382,7 @@ peas_object_module_new (const gchar *module_name,
  * @resident: If the module should be resident.
  * @local_linkage: Whether to load the module with local linkage.
  *
- * Creates a new #PeasObjectModule with local linkage.
+ * Creates a new #PeasObjectModule.
  *
  * Return value: a new #PeasObjectModule.
  *
@@ -356,6 +394,9 @@ peas_object_module_new_full (const gchar *module_name,
                              gboolean     resident,
                              gboolean     local_linkage)
 {
+  g_return_val_if_fail (module_name != NULL && *module_name != '\0', NULL);
+  g_return_val_if_fail (path != NULL && *path != '\0', NULL);
+
   return PEAS_OBJECT_MODULE (g_object_new (PEAS_TYPE_OBJECT_MODULE,
                                            "module-name", module_name,
                                            "path", path,
@@ -364,6 +405,31 @@ peas_object_module_new_full (const gchar *module_name,
                                            NULL));
 }
 
+/**
+ * peas_object_module_new_embedded: (skip)
+ * @module_name: The module name.
+ *
+ * Creates a new #PeasObjectModule for an embedded plugin.
+ *
+ * Return value: a new #PeasObjectModule.
+ *
+ * Since: 1.18
+ */
+PeasObjectModule *
+peas_object_module_new_embedded (const gchar *module_name,
+                                 const gchar *symbol)
+{
+  g_return_val_if_fail (module_name != NULL && *module_name != '\0', NULL);
+  g_return_val_if_fail (symbol != NULL && *symbol != '\0', NULL);
+
+  return PEAS_OBJECT_MODULE (g_object_new (PEAS_TYPE_OBJECT_MODULE,
+                                           "module-name", module_name,
+                                           "symbol", symbol,
+                                           "resident", TRUE,
+                                           "local-linkage", FALSE,
+                                           NULL));
+}
+
 /**
  * peas_object_module_create_object: (skip)
  * @module: A #PeasObjectModule.
@@ -390,10 +456,15 @@ peas_object_module_create_object (PeasObjectModule *module,
 
   g_return_val_if_fail (PEAS_IS_OBJECT_MODULE (module), NULL);
 
+  if (interface != PEAS_TYPE_PLUGIN_LOADER)
+    g_return_val_if_fail (G_TYPE_IS_INTERFACE (interface), NULL);
+
   impls = (InterfaceImplementation *) priv->implementations->data;
   for (i = 0; i < priv->implementations->len; ++i)
-    if (impls[i].iface_type == interface)
-      return impls[i].func (n_parameters, parameters, impls[i].user_data);
+    {
+      if (impls[i].iface_type == interface)
+        return impls[i].func (n_parameters, parameters, impls[i].user_data);
+    }
 
   return NULL;
 }
@@ -417,10 +488,15 @@ peas_object_module_provides_object (PeasObjectModule *module,
 
   g_return_val_if_fail (PEAS_IS_OBJECT_MODULE (module), FALSE);
 
+  if (interface != PEAS_TYPE_PLUGIN_LOADER)
+    g_return_val_if_fail (G_TYPE_IS_INTERFACE (interface), FALSE);
+
   impls = (InterfaceImplementation *) priv->implementations->data;
   for (i = 0; i < priv->implementations->len; ++i)
-    if (impls[i].iface_type == interface)
-      return TRUE;
+    {
+      if (impls[i].iface_type == interface)
+        return TRUE;
+    }
 
   return FALSE;
 }
@@ -461,6 +537,26 @@ peas_object_module_get_module_name (PeasObjectModule *module)
   return priv->module_name;
 }
 
+/**
+ * peas_object_module_get_symbol: (skip)
+ * @module: A #PeasObjectModule.
+ *
+ * Gets the symbol name used to register extension implementations.
+ *
+ * Return value: the symbol name.
+ *
+ * Since: 1.18
+ */
+const gchar *
+peas_object_module_get_symbol (PeasObjectModule *module)
+{
+  PeasObjectModulePrivate *priv = GET_PRIV (module);
+
+  g_return_val_if_fail (PEAS_IS_OBJECT_MODULE (module), NULL);
+
+  return priv->symbol;
+}
+
 /**
  * peas_object_module_get_library: (skip)
  * @module: A #PeasObjectModule.
@@ -524,28 +620,29 @@ create_gobject_from_type (guint       n_parameters,
                           GParameter *parameters,
                           gpointer    user_data)
 {
-  GType exten_type;
-  GObjectClass *cls;
-  GObject *instance;
+  GType exten_type = GPOINTER_TO_SIZE (user_data);
 
-  exten_type = GPOINTER_TO_SIZE (user_data);
+  /* We should be called with a "plugin-info" property appended
+   * to the parameters. Let's get rid of it if the actual type
+   * doesn't have such a property as it would cause a warning.
+   */
+  if ((exten_type & TYPE_MISSING_PLUGIN_INFO_PROPERTY) != 0)
+    {
+      exten_type &= ~TYPE_MISSING_PLUGIN_INFO_PROPERTY;
 
-  cls = g_type_class_ref (exten_type);
+      if (n_parameters > 0)
+        {
+          GParameter *info_param = &parameters[n_parameters - 1];
 
-  /* If we are instantiating a plugin, then the factory function is
-   * called with a "plugin-info" property appended to the parameters.
-   * Let's get rid of it if the actual type doesn't have such a
-   * property to avoid a warning. */
-  if (n_parameters > 0 &&
-      strcmp (parameters[n_parameters - 1].name, "plugin-info") == 0 &&
-      g_object_class_find_property (cls, "plugin-info") == NULL)
-    n_parameters--;
+          if (info_param->name == intern_plugin_info &&
+              G_VALUE_TYPE (&info_param->value) == PEAS_TYPE_PLUGIN_INFO)
+            {
+              n_parameters--;
+            }
+        }
+    }
 
-  instance = G_OBJECT (g_object_newv (exten_type, n_parameters, parameters));
-
-  g_type_class_unref (cls);
-
-  return instance;
+  return G_OBJECT (g_object_newv (exten_type, n_parameters, parameters));
 }
 
 /**
@@ -562,18 +659,28 @@ peas_object_module_register_extension_type (PeasObjectModule *module,
                                             GType             iface_type,
                                             GType             extension_type)
 {
+  GObjectClass *cls;
+  GParamSpec *pspec;
+
   g_return_if_fail (PEAS_IS_OBJECT_MODULE (module));
   g_return_if_fail (!peas_object_module_provides_object (module, iface_type));
+  g_return_if_fail (g_type_is_a (extension_type, iface_type));
 
   if (iface_type != PEAS_TYPE_PLUGIN_LOADER)
-    {
-      g_return_if_fail (G_TYPE_IS_INTERFACE (iface_type));
-      g_return_if_fail (g_type_is_a (extension_type, iface_type));
-    }
+    g_return_if_fail (G_TYPE_IS_INTERFACE (iface_type));
+
+  cls = g_type_class_ref (extension_type);
+  pspec = g_object_class_find_property (cls, "plugin-info");
+
+  /* Avoid checking for this each time in the factory function */
+  if (pspec == NULL || pspec->value_type != PEAS_TYPE_PLUGIN_INFO)
+    extension_type |= TYPE_MISSING_PLUGIN_INFO_PROPERTY;
 
   peas_object_module_register_extension_factory (module,
                                                  iface_type,
                                                  create_gobject_from_type,
                                                  GSIZE_TO_POINTER (extension_type),
                                                  NULL);
+
+  g_type_class_unref (cls);
 }
