@@ -35,7 +35,6 @@
 
 #include "introspection-base.h"
 #include "introspection-callable.h"
-#include "introspection-has-missing-prerequisite.h"
 #include "introspection-has-prerequisite.h"
 #include "introspection-unimplemented.h"
 
@@ -81,10 +80,17 @@ static void
 test_extension_garbage_collect (PeasEngine     *engine,
                                 PeasPluginInfo *info)
 {
+  gchar **loaded_plugins;
+
   peas_engine_garbage_collect (engine);
 
   /* Check that we can collect the garbage when no plugins are loaded */
   g_assert (peas_engine_unload_plugin (engine, info));
+
+  loaded_plugins = peas_engine_get_loaded_plugins (engine);
+  g_assert_cmpstr (loaded_plugins[0], ==, NULL);
+  g_strfreev (loaded_plugins);
+
   peas_engine_garbage_collect (engine);
 }
 
@@ -145,7 +151,7 @@ test_extension_create_invalid (PeasEngine     *engine,
 
   testing_util_push_log_hook ("*assertion*G_TYPE_IS_INTERFACE*failed");
   testing_util_push_log_hook ("*does not provide a 'IntrospectionUnimplemented' extension");
-  testing_util_push_log_hook ("*type 'IntrospectionCallable' has no property named 'invalid-property'");
+  testing_util_push_log_hook ("*type 'IntrospectionCallable' has no property named 'does-not-exist'");
   testing_util_push_log_hook ("*assertion*peas_plugin_info_is_loaded*failed");
 
   /* Invalid GType */
@@ -164,31 +170,13 @@ test_extension_create_invalid (PeasEngine     *engine,
                                             NULL);
   g_assert (!PEAS_IS_EXTENSION (extension));
 
-  /* Interface does not have an 'invalid-property' property */
+  /* Interface does not have a specified property */
   extension = peas_engine_create_extension (engine, info,
                                             INTROSPECTION_TYPE_CALLABLE,
-                                            "invalid-property", "does-not-exist",
+                                            "does-not-exist", "",
                                             NULL);
   g_assert (!PEAS_IS_EXTENSION (extension));
 
-  /* This cannot be tested in PyGI and Seed's log handler messes this up */
-  if (g_strcmp0 (extension_plugin, "extension-c") != 0 &&
-      g_strcmp0 (extension_plugin, "extension-lua51") != 0 &&
-      g_strcmp0 (extension_plugin, "extension-python") != 0 &&
-      g_strcmp0 (extension_plugin, "extension-python3") != 0)
-    {
-      testing_util_push_log_hook ("*cannot add *IntrospectionHasMissingPrerequisite* "
-                                  "which does not conform to *IntrospectionCallable*");
-      testing_util_push_log_hook ("*Type *HasMissingPrerequisite* is invalid");
-      testing_util_push_log_hook ("*does not provide a *HasMissingPrerequisite* extension");
-
-      /* Missing Prerequisite */
-      extension = peas_engine_create_extension (engine, info,
-                                                INTROSPECTION_TYPE_HAS_MISSING_PREREQUISITE,
-                                                NULL);
-      g_assert (!PEAS_IS_EXTENSION (extension));
-    }
-
   /* Not loaded */
   g_assert (peas_engine_unload_plugin (engine, info));
   extension = peas_engine_create_extension (engine, info,
